/*
  hci_csr.c

  csr specific initialization
*/
/* 
   BlueMP3 firmware (c) 2004 by Till Harbaum, harbaum@beecon.de
*/

/*
   Dieser Code ist Teil der BlueMP3-Firmware wie in den Ausgaben
   8, 9 und 10 2004 des Computermagazins c't beschrieben.

   Der Autor bernimmt keinerlei Garantie fr diesen Code. Dieser
   Code darf ohne Zustimmung des Autors in keiner Form weitervertrieben 
   werden.

   Dieser Code ist fr rein diadaktische Zwecke gedacht und ist 
   mglicherweise nicht kompatibel mit den Bluetooth-Standards.
*/

/*
   This code is part of the BlueMP3 firmware as published in issues
   8, 9, and 10 2004 of the german computer magazine c't. 

   The code comes without warantee of any kind, either expressed or
   implied. This code may not be distributed in any form without
   permission by the author.

   The code is provided for educational purposes only and may not comply 
   to the bluetooth standards.
*/

#include "types.h"
#include "utils.h"
#include "debug.h"

#include "hci.h"
#include "hci_uart.h"
#include "uart.h"
#include "hci_event.h"

#include "hci_csr.h"

#define HCI_CSR_STATE_REQ        0
#define HCI_CSR_STATE_REPLY_OK   1
#define HCI_CSR_STATE_REPLY_ERR  2

u08_t hci_csr_state;

#define HCI_CSR_CMD_HDR(a)  OPCODE(OGF_VENDOR, 0), 11+2*(a), 0xc2

/* scelleton of a hci csr vendor message */
u08_t hci_cmd_get_set_config_uart[] = {
  HCI_CSR_CMD_HDR(4),

  MK_SEQ2(HCI_CSR_GET),
  MK_SEQ2(9),      /* length */
  MK_SEQ2(0x1235), /* just a sequence number */

  MK_SEQ2(CSR_CMD_CONFIG_UART),
  MK_SEQ2(0),      /* status */

  MK_SEQ2(0), MK_SEQ2(0), MK_SEQ2(0), MK_SEQ2(0)
};

void hci_csr_send(u08_t *data, u08_t len) {
  /* stop receiving data if we are going to switch baud rate */
  if(data[4] == HCI_CSR_SET) uart_enable_rx(FALSE);

  /* we are now waiting for the right event */
  hci_csr_state = HCI_CSR_STATE_REQ;

  hci_uart_put(HCI_CMD);   // this will be a hci command
  hci_uart_send(data, len);

  if(data[4] == HCI_CSR_SET) {
    /* change the speed, since the rate changes */
    uart_set_bitrate();
    /* and re-enable receiver */
    uart_enable_rx(TRUE);
  }

  /* wait until we have a result via a vendor event */
  while(hci_csr_state == HCI_CSR_STATE_REQ)
    hci_process(HCI_PROCESS_IDLE);
}

void hci_csr_init(void) {
  /* wait until we get ok to send (further) commands */
  while(!hci_further_commands) hci_process(HCI_PROCESS_WAIT_CMD);

  /* get the uart speed */
  hci_csr_send(hci_cmd_get_set_config_uart, 
	       sizeof(hci_cmd_get_set_config_uart));

  if(hci_csr_state == HCI_CSR_STATE_REPLY_OK) {
    DEBUG_HCI("valid 115k baud rate detected, changing it");

    /* transform get uart command into set uart command */
    hci_cmd_get_set_config_uart[4]  = HCI_CSR_SET;
    hci_cmd_get_set_config_uart[14] = CSR_BAUD2FACTOR(HCI_BAUDRATE) & 0xff;
    hci_cmd_get_set_config_uart[15] = CSR_BAUD2FACTOR(HCI_BAUDRATE) >> 8;

    hci_csr_send(hci_cmd_get_set_config_uart, 
		 sizeof(hci_cmd_get_set_config_uart));
  } else {
    DEBUG_HCI("invalid baud rate setting detected");
  }
}

void hci_csr_event(u08_t len) {

  DEBUG_HCI("csr vendor specific event");

  hci_uart_drop(7);   // skip all kinds of unwanted header 

  switch(hci_uart_get_u16()) {

    case CSR_CMD_CONFIG_UART:
      hci_uart_drop(2);
      DEBUG_HCI("got uart config"); 
      /* verify if this is 115k setting */
      hci_csr_state = (hci_uart_get_u16() == CSR_BAUD2FACTOR(115200l))?
	             HCI_CSR_STATE_REPLY_OK:HCI_CSR_STATE_REPLY_ERR;
      hci_uart_drop(len-13);
      break;

    default:
      DEBUG_HCI("unknown csr vendor event");
      hci_uart_drop(len-9);      
  }
}
