/*
  hci_uart.c

  hci specific helper functions for uart operation
*/
/* 
   BlueMP3 firmware (c) 2004 by Till Harbaum, harbaum@beecon.de
*/

/*
   Dieser Code ist Teil der BlueMP3-Firmware wie den den Ausgaben
   8, 9 und 10 2004 des Computermagazins c't.

   Der Autor bernimmt keinerlei Garantie fr diesen Code. Dieser
   Code darf ohne Zustimmung des Autors in keiner Form weitervertrieben 
   werden.

   Dieser Code ist fr rein diadaktische Zwecke gedacht und ist 
   mglicherweise nicht kompatibel mit den Bluetooth-Standards.
*/

/*
   This code is part of the BlueMP3 firmware as published in issues
   8, 9, and 10 2004 of the german computer magazine c't. 

   The code comes without warantee of any kind, either expressed or
   implied. This code may not be distributed in any form without
   permission by the author.

   The code is provided for educational purposes only and may not comply 
   to the bluetooth standards.
*/

#include "types.h"
#include "uart.h"
#include "debug.h"
#include "hci_uart.h"

#ifdef DEBUG_UART
/* print a hex number on the debug interface */
void put_hex(bool_t in, u08_t val) {
  u08_t n;

  if(in) console_putc('>');
  else   console_putc('<');

  n = val>>4;
  if(n<10) console_putc('0'+n);
  else     console_putc('a'+n-10);

  n = val&0x0f;
  if(n<10) console_putc('0'+n);
  else     console_putc('a'+n-10);

  console_putc(' ');
}
#endif

void hci_uart_put(u08_t val) {
#ifdef DEBUG_UART
  put_hex(FALSE, val);
#endif
  uart_putc(UART1, val);
}

/* just echo a number of bytes */
void hci_uart_echo(u16_t len) {
  while(len--)
    hci_uart_put(hci_uart_get());
}

/* write a stream of bytes to the uart */
void hci_uart_send(u08_t *data, u16_t len) {
  while(len--) hci_uart_put(*data++);
}

/* send a 16 bit value */
void hci_uart_put_u16(u16_t val) {
  hci_uart_put(val  & 0xff);
  hci_uart_put(val >> 8);
}

/* write a stream of bytes to the uart */
void hci_uart_send_zero(u16_t len) {
  while(len--) hci_uart_put(0);
}

u08_t hci_uart_get(void) {
#ifdef DEBUG_UART
  u08_t i;

  while(!uart_input_avail());

  i = uart_getc();
  put_hex(TRUE, i);
  return i;
#else
  while(!uart_input_avail());
  return uart_getc();
#endif
}

/* receive several bytes, block if no byte received yet */
void hci_uart_get_n(u08_t *data, u16_t len) {
  while(len--)
   *data++ = hci_uart_get();
}

/* drop bytes from input buffer */
void hci_uart_drop(u16_t bytes) {
  while(bytes--) hci_uart_get();
}

/* read two bytes and interprete them as a 16 bit value */
u16_t hci_uart_get_u16(void) {
  return hci_uart_get() + 256 * hci_uart_get();
}
