/*
  vs1001.h
*/
/* 
   BlueMP3 firmware (c) 2004 by Till Harbaum, harbaum@beecon.de
*/

/*
   Dieser Code ist Teil der BlueMP3-Firmware wie den den Ausgaben
   8, 9 und 10 2004 des Computermagazins c't.

   Der Autor bernimmt keinerlei Garantie fr diesen Code. Dieser
   Code darf ohne Zustimmung des Autors in keiner Form weitervertrieben 
   werden.

   Dieser Code ist fr rein diadaktische Zwecke gedacht und ist 
   mglicherweise nicht kompatibel mit den Bluetooth-Standards.
*/

/*
   This code is part of the BlueMP3 firmware as published in issues
   8, 9, and 10 2004 of the german computer magazine c't. 

   The code comes without warantee of any kind, either expressed or
   implied. This code may not be distributed in any form without
   permission by the author.

   The code is provided for educational purposes only and may not comply 
   to the bluetooth standards.
*/

#ifndef VS1001_H
#define VS1001_H

#include "types.h"
#include "debug.h"

#include "l2cap.h"

/* some system may use 12 mhz */
#define VS1001_CLOCK_12MHZ  12000000
#define VS1001_CLOCK_24MHZ  24576000

// mp3 specific port settings
#define VS1001_XCS_PORT       PORTB
#define VS1001_XCS_DDR        DDRB
#define VS1001_XCS_BIT        (6)    // chip select
 
#define VS1001_BSYNC_PORT     PORTB    
#define VS1001_BSYNC_DDR      DDRB    
#define VS1001_BSYNC_BIT      (7)   // BSYNC signal
 
#define VS1001_RESET_PORT     PORTB
#define VS1001_RESET_DDR      DDRB
#define VS1001_RESET_BIT      (5)   // reset signal
 
#define VS1001_DREQ_PORT      PINE
#define VS1001_DREQ_DDR       DDRE
#define VS1001_DREQ_BIT       (4)   // DREQ signal
#define VS1001_DREQ_INT       INT4
#define VS1001_DREQ_SIG       SIG_INTERRUPT4

/* vs1001 commands */
#define VS1001_WRITE	     2
#define VS1001_READ	     3

/* vs1001 register names */
#define VS1001_MODE          0
#define VS1001_STATUS        1
#define VS1001_RES0          2
#define VS1001_CLOCKF        3
#define VS1001_DECODE_TIME   4
#define VS1001_AUDATA        5
#define VS1001_WRAM          6
#define VS1001_WRAMADDR      7
#define VS1001_HDAT0         8
#define VS1001_HDAT1         9
#define VS1001_AIADDR       10
#define VS1001_VOL          11
#define VS1001_RES1         12

/* mode bits */
#define VS1001_SM_DIFF      0x0001   // differential right/left
#define VS1001_SM_FFWD      0x0002   // fast forward (header only proc.)
#define VS1001_SM_RESET     0x0004   // soft reset
#define VS1001_SM_MP12      0x0008   // play mpeg 1 and 2 as well
#define VS1001_SM_PDOWN     0x0010   // power down
#define VS1001_SM_DAC       0x0020   // enable dac mode
#define VS1001_SM_DACMONO   0x0040   // stereo/mono dac
#define VS1001_SM_BASS      0x0080   // bass/treble enhancer
#define VS1001_SM_DACT      0x0100   // dclk active edge
#define VS1001_SM_BYTEORD   0x0200   // byte order on spi
#define VS1001_SM_IBMODE    0x0400   // input bus mode
#define VS1001_SM_IBCLK     0x0800   // master clock

bool_t vs1001_init(void);
void   vs1001_soft_reset(void);
void   vs1001_hard_reset(void);

/* read and write control registers */
u16_t vs1001_read(u08_t address);
void  vs1001_write(u08_t address, u16_t value);
void  vs1001_receive(hci_l2con_t *l2con, u16_t len);

/* upper layer functions */
void  vs1001_set_volume(u08_t left, u08_t right);

void  vs1001_zero(u08_t num);

#endif // VS1001_H
